<?php
/**
 * Card Gate Payment Gateway
 *
 * @author Card Gate B.V.
 * @package class example
 * @version 1.0
 *
 * == BEGIN LICENSE ==
 *
 * THIS SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT OF THIRD PARTY RIGHTS.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE
 * OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * == END LICENSE ==
 */

class CardGate {

	# These settings are given in the contstructor and can be retrieved in the
	# CG backoffice (https://merchants.cardgateplus.com)
	protected $_iSiteID;
	protected $_sAPIKey;
	protected $_bTestMode;

	public $sLastError;

	/**
     * Class Constructor
     * @param Integer $iSiteID_ Site id you wish to do payments for
     * @param String  $sAPIKey_ Shared secret for hash calculation
     * @param Boolean $bTestMode_ Use test- or live modus
     */
	public function __construct( $iSiteID_, $sAPIKey_, $bTestMode_ = FALSE ) {
		$this->_iSiteID = $iSiteID_;
		$this->_sAPIKey = $sAPIKey_;
		$this->_bTestMode = $bTestMode_;
	}

	/**
     * Hash Calculator
     * @param Integer $iAmount_ Transaction amount in cents
     * @param String  $sReference_ Local (unique!) transaction reference
     * @return String Hash value
     */
	public function calculateHash( $iAmount_, $sReference_ ) {
		return md5(
			( $this->_bTestMode ? 'TEST' : '' ) 	// Prefix for test transactions
			. $this->_iSiteID
			. $iAmount_
			. $sReference_
			. $this->_sAPIKey
			);
	}

	/**
     * Getters for settings
     */
	public function getSiteID() {
		return $this->_iSiteID;
	}

	public function getAPIKey() {
		return $this->_sAPIKey;
	}

	public function getTestMode() {
		return $this->_bTestMode;
	}

	/**
     * Default gateway URL
     * @return String URL
     */
	public function getGatewayURL() {
		return 'https://gateway.cardgateplus.com/';
	}

	/**
     * Check CardGate callback arguments and hash
     * @return Boolean check result
     */
	public function checkCallback( $aRequest_ ) {
		// Check required variables
		foreach( array( 'transaction_id', 'currency', 'amount', 'ref', 'status' ) as $sRequired ) {
			if( ! isset( $aRequest_[$sRequired] ) ) {
				$this->sLastError = 'Missing required ' .$sRequired;
				return FALSE;
			}
		}
		// Calculate callback hash
		$sHash = $this->calculateCallbackHash( $aRequest_ );
		if( $aRequest_['hash'] !== $sHash ) {
			$this->sLastError = 'Invalid hash ' . "$sHash -> {$aRequest_['is_test']} {$aRequest_['transaction_id']} {$aRequest_['currency']} {$aRequest_['amount']} {$aRequest_['ref']} {$aRequest_['status']} $this->_sAPIKey";
			return FALSE;
		}
		$this->sLastError = '';
		return TRUE;
	}

	/**
     * Hash Calculator for Callbacks
     * @return String Hash value
     */
	public function calculateCallbackHash( $aRequest_ ) {
		return md5(
			(  ! empty( $aRequest_['is_test'] ) ? 'TEST' : '' )
			. $aRequest_['transaction_id']
			. $aRequest_['currency']
			. $aRequest_['amount']
			. $aRequest_['ref']
			. $aRequest_['status']
			. $this->_sAPIKey
			);
	}

	/**
     * Get iDEAL bank list
     * @return Array List of current banks
     */
	public function getiDEALBanks() {
		$sBankOptions = file_get_contents( 'https://gateway.cardgateplus.com/cache/idealDirectoryCUROPayments.dat' );
		if ( empty( $sBankOptions ) || $sBankOptions[ 0 ] != 'a' ) {
			# Fallback in case of an error
			$sBankOptions = 'a:10:{i:0;s:0:"";s:8:"ABNANL2A";s:8:"ABN Amro";s:8:"INGBNL2A";s:3:"ING";s:8:"RABONL2U";s:8:"Rabobank";s:8:"SNSBNL2A";s:8:"SNS Bank";s:8:"ASNBNL21";s:8:"ASN Bank";s:8:"KNABNL2H";s:4:"Knab";s:8:"RBRBNL21";s:9:"RegioBank";s:8:"TRIONL2U";s:12:"Triodos Bank";s:8:"FVLBNL22";s:21:"Van Lanschot Bankiers";};';
		}
		$aBankOptions = unserialize( $sBankOptions );
		unset( $aBankOptions[ 0 ] ); # Remove blank option
		return $aBankOptions;
	}

	/**
     * Get iDEAL bank selectbox
     * @return String HTML select with iDEAL banks
     */
	public function getiDEALBanksSelect() {
		# Convert to HTML
		$sBankOptions = '<optgroup label="Kies uw bank">';
		foreach ( $this->getiDEALBanks() as $k => $v ) {
			if ( $v[ 0 ] == '-' ) {
				$sBankOptions .= '<optgroup label="' . str_replace( '-', '', $v ) . '">';
			} else {
				$sBankOptions .= '<option value="' . $k . '">' . $v . '</option>';
			}
		}
		$sBankOptions.= '</optgroup>';

		return $sBankOptions;
	}
}
// == End of class CardGate